@@ -20,86 +20,14 @@ else:
     ABC = abc.ABCMeta('ABC', (), {})
 
 
-class ActiveLearner(BaseEstimator):
-    """
-    This class is an abstract model of a general active learning algorithm.
-
-    Parameters
-    ----------
-    estimator: scikit-learn estimator
-        The estimator to be used in the active learning loop.
-
-    query_strategy: function
-        Function providing the query strategy for the active learning
-        loop, for instance modAL.uncertainty.uncertainty_sampling.
-
-    X_training: None or numpy.ndarray of shape (n_samples, n_features)
-        Initial training samples, if available.
-
-    y_training: None or numpy.ndarray of shape (n_samples, )
-        Initial training labels corresponding to initial training samples
-
-    bootstrap_init: boolean
-        If initial training data is available, bootstrapping can be done
-        during the first training. Useful when building Committee models
-        with bagging.
-
-    fit_kwargs: keyword arguments for the fit method
-
-    Attributes
-    ----------
-    estimator: scikit-learn estimator
-        The estimator to be used in the active learning loop.
-
-    query_strategy: function
-        Function providing the query strategy for the active learning
-        loop, for instance modAL.query.max_uncertainty.
-
-    X_training: None numpy.ndarray of shape (n_samples, n_features)
-        If the model hasn't been fitted yet: None
-        If the model has been fitted already: numpy.ndarray containing the
-        samples which the model has been trained on
-
-    y_training: None or numpy.ndarray of shape (n_samples, )
-        If the model hasn't been fitted yet: None
-        If the model has been fitted already: numpy.ndarray containing the
-        labels corresponding to _training_samples
-
-    Examples
-    --------
-    >>> from sklearn.datasets import load_iris
-    >>> from sklearn.ensemble import RandomForestClassifier
-    >>> from modAL.models import ActiveLearner
-    >>>
-    >>> iris = load_iris()
-    >>> # give initial training examples
-    >>> X_training = iris['data'][[0, 50, 100]]
-    >>> y_training = iris['target'][[0, 50, 100]]
-    >>>
-    >>> # initialize active learner
-    >>> learner = ActiveLearner(
-    ...     estimator=RandomForestClassifier(),
-    ...     X_training=X_training, y_training=y_training
-    ... )
-    >>>
-    >>> # querying for labels
-    >>> query_idx, query_sample = learner.query(iris['data'])
-    >>>
-    >>> # ...obtaining new labels from the Oracle...
-    >>>
-    >>> # teaching newly labelled examples
-    >>> learner.teach(
-    ...     X=iris['data'][query_idx].reshape(1, -1),
-    ...     y=iris['target'][query_idx].reshape(1, )
-    ... )
-    """
+class BaseLearner(ABC, BaseEstimator):
     def __init__(
             self,
-            estimator,                                           # scikit-learner estimator object
-            query_strategy=uncertainty_sampling,	             # callable to query labels
-            X_training=None, y_training=None,	                 # initial data if available
-            bootstrap_init=False,                                # first training with bootstrapping
-            **fit_kwargs                                         # keyword arguments for fitting the initial data
+            estimator,                            # scikit-learner estimator object
+            query_strategy=uncertainty_sampling,  # callable to query labels
+            X_training=None, y_training=None,     # initial data if available
+            bootstrap_init=False,                 # first training with bootstrapping
+            **fit_kwargs                          # keyword arguments for fitting the initial data
     ):
         assert callable(query_strategy), 'query_function must be callable'
 
@@ -163,12 +91,12 @@ class ActiveLearner(BaseEstimator):
         """
         if not bootstrap:
             self.estimator.fit(self.X_training, self.y_training, **fit_kwargs)
-            return self
         else:
             n_instances = len(self.X_training)
             bootstrap_idx = np.random.choice(range(n_instances), n_instances, replace=True)
             self.estimator.fit(self.X_training[bootstrap_idx], self.y_training[bootstrap_idx], **fit_kwargs)
-            return self
+
+        return self
 
     def _fit_on_new(self, X, y, bootstrap=False, **fit_kwargs):
         """
@@ -193,11 +121,11 @@ class ActiveLearner(BaseEstimator):
 
         if not bootstrap:
             self.estimator.fit(X, y, **fit_kwargs)
-            return self
         else:
             bootstrap_idx = np.random.choice(range(len(X)), len(X), replace=True)
             self.estimator.fit(X[bootstrap_idx], y[bootstrap_idx])
-            return self
+
+        return self
 
     def fit(self, X, y, bootstrap=False, **fit_kwargs):
         """
@@ -227,8 +155,7 @@ class ActiveLearner(BaseEstimator):
         """
         self.X_training = X
         self.y_training = y
-        self._fit_to_known(bootstrap=bootstrap, **fit_kwargs)
-        return self
+        return self._fit_to_known(bootstrap=bootstrap, **fit_kwargs)
 
     def predict(self, X, **predict_kwargs):
         """
@@ -295,7 +222,7 @@ class ActiveLearner(BaseEstimator):
             The instances from X_pool chosen to be labelled.
         """
 
-        query_idx, query_instances = self.query_strategy(self.estimator, X, **query_kwargs)
+        query_idx, query_instances = self.query_strategy(self, X, **query_kwargs)
         return query_idx, query_instances
 
     def score(self, X, y, **score_kwargs):
@@ -320,6 +247,85 @@ class ActiveLearner(BaseEstimator):
         """
         return self.estimator.score(X, y, **score_kwargs)
 
+    @abc.abstractmethod
+    def teach(self, *args, **kwargs):
+        pass
+
+
+class ActiveLearner(BaseLearner):
+    """
+    This class is an abstract model of a general active learning algorithm.
+
+    Parameters
+    ----------
+    estimator: scikit-learn estimator
+        The estimator to be used in the active learning loop.
+
+    query_strategy: function
+        Function providing the query strategy for the active learning
+        loop, for instance modAL.uncertainty.uncertainty_sampling.
+
+    X_training: None or numpy.ndarray of shape (n_samples, n_features)
+        Initial training samples, if available.
+
+    y_training: None or numpy.ndarray of shape (n_samples, )
+        Initial training labels corresponding to initial training samples.
+
+    bootstrap_init: boolean
+        If initial training data is available, bootstrapping can be done
+        during the first training. Useful when building Committee models
+        with bagging.
+
+    fit_kwargs: keyword arguments for the fit method
+
+    Attributes
+    ----------
+    estimator: scikit-learn estimator
+        The estimator to be used in the active learning loop.
+
+    query_strategy: function
+        Function providing the query strategy for the active learning
+        loop, for instance modAL.query.max_uncertainty.
+
+    X_training: None numpy.ndarray of shape (n_samples, n_features)
+        If the model hasn't been fitted yet: None
+        If the model has been fitted already: numpy.ndarray containing the
+        samples which the model has been trained on
+
+    y_training: None or numpy.ndarray of shape (n_samples, )
+        If the model hasn't been fitted yet: None
+        If the model has been fitted already: numpy.ndarray containing the
+        labels corresponding to _training_samples
+
+    Examples
+    --------
+    >>> from sklearn.datasets import load_iris
+    >>> from sklearn.ensemble import RandomForestClassifier
+    >>> from modAL.models import ActiveLearner
+    >>>
+    >>> iris = load_iris()
+    >>> # give initial training examples
+    >>> X_training = iris['data'][[0, 50, 100]]
+    >>> y_training = iris['target'][[0, 50, 100]]
+    >>>
+    >>> # initialize active learner
+    >>> learner = ActiveLearner(
+    ...     estimator=RandomForestClassifier(),
+    ...     X_training=X_training, y_training=y_training
+    ... )
+    >>>
+    >>> # querying for labels
+    >>> query_idx, query_sample = learner.query(iris['data'])
+    >>>
+    >>> # ...obtaining new labels from the Oracle...
+    >>>
+    >>> # teaching newly labelled examples
+    >>> learner.teach(
+    ...     X=iris['data'][query_idx].reshape(1, -1),
+    ...     y=iris['target'][query_idx].reshape(1, )
+    ... )
+    """
+
     def teach(self, X, y, bootstrap=False, only_new=False, **fit_kwargs):
         """
         Adds X and y to the known training data and retrains the predictor
@@ -328,8 +334,7 @@ class ActiveLearner(BaseEstimator):
         Parameters
         ----------
         X: numpy.ndarray of shape (n_samples, n_features)
-            The new samples for which the labels are supplied
-            by the expert.
+            The new samples for which the labels are supplied by the expert.
 
         y: numpy.ndarray of shape (n_samples, )
             Labels corresponding to the new instances in X.
@@ -344,8 +349,7 @@ class ActiveLearner(BaseEstimator):
             doesn't retrain the model from scratch. (For example, in tensorflow or keras.)
 
         fit_kwargs: keyword arguments
-            Keyword arguments to be passed to the fit method
-            of the predictor.
+            Keyword arguments to be passed to the fit method of the predictor.
         """
         self._add_training_data(X, y)
         if not only_new:
@@ -354,6 +358,166 @@ class ActiveLearner(BaseEstimator):
             self._fit_on_new(X, y, bootstrap=bootstrap, **fit_kwargs)
 
 
+class BayesianOptimizer(BaseLearner):
+    """
+    This class is an abstract model of a Bayesian optimizer algorithm.
+
+    Parameters
+    ----------
+    estimator: scikit-learn regressor
+        The regressor to be used in the Bayesian optimization algorithm.
+
+    query_strategy: function
+        Function providing the query strategy for the Bayesian optimization
+        loop, for instance modAL.acquisition.max_PI.
+
+    X_training: None or numpy.ndarray of shape (n_samples, n_features)
+        Initial training samples, if available.
+
+    y_training: None or numpy.ndarray of shape (n_samples, )
+        Initial values corresponding to initial training samples.
+
+    bootstrap_init: boolean
+        If initial training data is available, bootstrapping can be done
+        during the first training.
+
+    fit_kwargs: keyword arguments for the fit method
+
+    Attributes
+    ----------
+    estimator: scikit-learn regressor
+        The regressor to be used in the Bayesian optimization algorithm.
+
+    query_strategy: function
+        Function providing the query strategy for the Bayesian optimization
+        loop, for instance modAL.acquisition.max_PI.
+
+    X_training: None numpy.ndarray of shape (n_samples, n_features)
+        If the model hasn't been fitted yet: None
+        If the model has been fitted already: numpy.ndarray containing the
+        samples which the model has been trained on
+
+    y_training: None or numpy.ndarray of shape (n_samples, )
+        If the model hasn't been fitted yet: None
+        If the model has been fitted already: numpy.ndarray containing the
+        labels corresponding to _training_samples
+
+    X_max: None or numpy.ndarray of shape (n_samples, 3)
+
+    y_max: -np.inf or float
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from functools import partial
+    >>> from sklearn.gaussian_process import GaussianProcessRegressor
+    >>> from sklearn.gaussian_process.kernels import Matern
+    >>> from modAL.models import BayesianOptimizer
+    >>> from modAL.acquisition import optimizer_PI, optimizer_EI, optimizer_UCB, max_PI, max_EI, max_UCB
+    >>>
+    >>> # generating the data
+    >>> X = np.linspace(0, 20, 1000).reshape(-1, 1)
+    >>> y = np.sin(X)/2 - ((10 - X)**2)/50 + 2
+    >>>
+    >>> # assembling initial training set
+    >>> X_initial, y_initial = X[150].reshape(1, -1), y[150].reshape(1, -1)
+    >>>
+    >>> # defining the kernel for the Gaussian process
+    >>> kernel = Matern(length_scale=1.0)
+    >>>
+    >>> tr = 0.1
+    >>> PI_tr = partial(optimizer_PI, tradeoff=tr)
+    >>> PI_tr.__name__ = 'PI, tradeoff = %1.1f' % tr
+    >>> max_PI_tr = partial(max_PI, tradeoff=tr)
+    >>>
+    >>> acquisitions = zip(
+    ...     [PI_tr, optimizer_EI, optimizer_UCB],
+    ...     [max_PI_tr, max_EI, max_UCB],
+    ... )
+    >>>
+    >>> for acquisition, query_strategy in acquisitions:
+    ...     # initializing the optimizer
+    ...     optimizer = BayesianOptimizer(
+    ...         estimator=GaussianProcessRegressor(kernel=kernel),
+    ...         X_training=X_initial, y_training=y_initial,
+    ...         query_strategy=query_strategy
+    ...     )
+    ...
+    ...     for n_query in range(5):
+    ...         # query
+    ...         query_idx, query_inst = optimizer.query(X)
+    ...         optimizer.teach(X[query_idx].reshape(1, -1), y[query_idx].reshape(1, -1))
+    """
+
+    def __init__(self, *args, **kwargs):
+        super(BayesianOptimizer, self).__init__(*args, **kwargs)
+        # setting the maximum value
+        if self.y_training is not None:
+            max_idx = np.argmax(self.y_training)
+            self.X_max = self.X_training[max_idx]
+            self.y_max = self.y_training[max_idx]
+        else:
+            self.X_max = None
+            self.y_max = -np.inf
+
+    def _set_max(self, X, y):
+        max_idx = np.argmax(y)
+        y_max = y[max_idx]
+        if y_max > self.y_max:
+            self.y_max = y_max
+            self.X_max = X[max_idx]
+
+    def get_max(self):
+        """
+        Gives the highest value so far.
+
+        Returns
+        -------
+        X: np.ndarray of shape (n_features)
+            The location of the currently best value.
+
+        y: np.ndarray of shape (1)
+            The currently best value.
+
+        """
+
+        return self.X_max, self.y_max
+
+    def teach(self, X, y, bootstrap=False, only_new=False, **fit_kwargs):
+        """
+        Adds X and y to the known training data and retrains the predictor with the
+        augmented dataset. This method also keeps track of the maximum value encountered
+        in the training data.
+
+        Parameters
+        ----------
+        X: numpy.ndarray of shape (n_samples, n_features)
+            The new samples for which the values are supplied.
+
+        y: numpy.ndarray of shape (n_samples, )
+            Values corresponding to the new instances in X.
+
+        bootstrap: boolean
+            If True, training is done on a bootstrapped dataset. Useful for building
+            Committee models with bagging.
+
+        only_new: boolean
+            If True, the model is retrained using only X and y, ignoring the previously
+            provided examples. Useful when working with models where the .fit() method
+            doesn't retrain the model from scratch. (For example, in tensorflow or keras.)
+
+        fit_kwargs: keyword arguments
+            Keyword arguments to be passed to the fit method of the predictor.
+        """
+        self._add_training_data(X, y)
+        if not only_new:
+            self._fit_to_known(bootstrap=bootstrap, **fit_kwargs)
+        else:
+            self._fit_on_new(X, y, bootstrap=bootstrap, **fit_kwargs)
+
+        self._set_max(X, y)
+
+
 class BaseCommittee(ABC, BaseEstimator):
     def __init__(
             self,
@@ -462,6 +626,8 @@ class BaseCommittee(ABC, BaseEstimator):
         for learner in self.learner_list:
             learner.fit(X, y, **fit_kwargs)
 
+        return self
+
     def query(self, X, **query_kwargs):
         """
         Finds the n_instances most informative point in the data provided by calling
@@ -708,7 +874,6 @@ class Committee(BaseCommittee):
         -------
         vote_proba: numpy.ndarray of shape (n_samples, n_learners, n_classes)
             Probabilities of each class for each learner and each instance.
-
         """
 
         # get dimensions
